import json

from django.contrib.auth.decorators import login_required
from django.http import JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View
from rest_framework_simplejwt.exceptions import InvalidToken, TokenError
from rest_framework_simplejwt.serializers import (
    TokenObtainPairSerializer,
    TokenRefreshSerializer,
    TokenVerifySerializer,
)


@login_required
def secret_data(request):
    return JsonResponse({"Hello": f"Hello {request.user.email}"})


class TokenView(View):
    serializer_class = None

    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        data = request.POST or json.loads(request.body)
        serializer = self.serializer_class(data=data)

        try:
            serializer.is_valid(raise_exception=True)
        except TokenError as e:
            raise InvalidToken(e.args[0])

        return JsonResponse(serializer.validated_data)


class TokenObtainPairView(TokenView):
    """
    Takes a set of user credentials and returns an access and refresh JSON web
    token pair to prove the authentication of those credentials.
    """

    serializer_class = TokenObtainPairSerializer


class TokenRefreshView(TokenView):
    """
    Takes a refresh type JSON web token and returns an access type JSON web
    token if the refresh token is valid.
    """

    serializer_class = TokenRefreshSerializer


class TokenVerifyView(TokenView):
    """
    Takes a token and indicates if it is valid.  This view provides no
    information about a token's fitness for a particular use.
    """

    serializer_class = TokenVerifySerializer



from django.urls import path

from .views import TokenObtainPairView, TokenRefreshView, TokenVerifyView

urlpatterns = [
    # Exclude as not appropriate for this app?
    # path('admin/', admin.site.urls),
    path("authenticate/", TokenObtainPairView.as_view(), name="authenticate"),
    path("refresh/", TokenRefreshView.as_view(), name="refresh"),
    path("verify/", TokenVerifyView.as_view(), name="verify"),
]



from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.utils.deprecation import MiddlewareMixin

from rest_framework_simplejwt.authentication import JWTAuthentication


class DjangoJWTAuthentication(JWTAuthentication, MiddlewareMixin):
    def __call__(self, request):
        if hasattr(request, "user") and not isinstance(request.user, AnonymousUser):
            return self.get_response(request)

        auth_user = self.authenticate(request)
        if isinstance(auth_user, tuple):
            request.user = auth_user[0]
        elif auth_user:
            request.user = auth_user
        else:
            request.user = AnonymousUser()

        return self.get_response(request)



from setuptools import find_packages, setup

from djjwt import __version__ as VERSION

PACKAGE_NAME = "dj-jwt"

with open("README.md", "r") as fh:
    long_description = fh.read()


def parse_requirements(filename):
    """Load requirements from a pip requirements file."""
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]


if __name__ == "__main__":
    setup(
        name=PACKAGE_NAME,
        version=VERSION,
        packages=find_packages(),
        install_requires=parse_requirements("requirements/base.txt"),
        url="https://github.com/buddylindsey/dj-jwt",
        author="Buddy Lindsey",
        author_email="buddy@buddylindsey.com",
        maintainer="Buddy Lindsey",
        description="Library to give vanilla Django jwt auth.",
        long_description=long_description,
        long_description_content_type="text/markdown",
        python_requires=">=3.7",
        classifiers=[
            "Development Status :: 4 - Beta",
            "Intended Audience :: Developers",
            "Natural Language :: English",
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Framework :: Django",
            "Framework :: Django :: 3.0",
        ],
        project_urls={
            "Documentation": "https://github.com/buddylindsey/dj-jwt",
            "Source": "https://github.com/buddylindsey/dj-jwt",
        },
    )
